@@ -8,9 +8,10 @@ Convenience utility functions for NiPyApi, not really intended for external use
 from __future__ import absolute_import, unicode_literals
 import logging
 import json
+import re
 import time
 from copy import copy
-from functools import reduce
+from functools import reduce, wraps
 import operator
 from contextlib import contextmanager
 from packaging import version
@@ -20,31 +21,42 @@ import docker
 from docker.errors import ImageNotFound
 import requests
 from requests.models import Response
+from future.utils import raise_from as _raise
 import nipyapi
 
 __all__ = ['dump', 'load', 'fs_read', 'fs_write', 'filter_obj',
            'wait_to_complete', 'is_endpoint_up', 'set_endpoint',
            'start_docker_containers', 'DockerContainer',
-           'infer_object_label_from_class', 'bypass_slash_encoding']
+           'infer_object_label_from_class', 'bypass_slash_encoding',
+           'exception_handler', 'enforce_min_ver', 'check_version'
+           ]
 
 log = logging.getLogger(__name__)
 
 
 def dump(obj, mode='json'):
     """
-    Dumps a native datatype object to json or yaml, defaults to json
+    Dumps a native datatype object or swagger entity to json or yaml, defaults to json
 
     Args:
-        obj (varies): The native datatype object to serialise
+        obj (varies): The native datatype object or swagger type to serialise
         mode (str): 'json' or 'yaml', the supported export modes
 
     Returns (str): The serialised object
 
     """
     assert mode in ['json', 'yaml']
+    unset = False
+    if nipyapi.config.nifi_config.api_client is None:
+        unset = True
+        nipyapi.config.nifi_config.api_client = nipyapi.nifi.ApiClient()
+
+    prepared_obj = nipyapi.config.nifi_config.api_client.sanitize_for_serialization(obj)
+    if unset:
+        nipyapi.config.nifi_config.api_client = None
     try:
         out = json.dumps(
-            obj=obj,
+            obj=prepared_obj,
             sort_keys=True,
             indent=4
             # default=_json_default
@@ -165,9 +177,11 @@ def filter_obj(obj, value, key, greedy=True):
         return None
     try:
         obj_class_name = obj[0].__class__.__name__
-    except (TypeError, IndexError):
-        raise TypeError("The passed object {0} is not a known filterable"
-                        " nipyapi object".format(obj.__class__.__name__))
+    except (TypeError, IndexError) as e:
+        _raise(
+            TypeError(
+                "The passed object {0} is not a filterable nipyapi object"
+                .format(obj.__class__.__name__)), e)
     # Check if this class has a registered filter in Nipyapi.config
     this_filter = nipyapi.config.registered_filters.get(obj_class_name, False)
     if not this_filter:
@@ -353,9 +367,11 @@ class DockerContainer:
             return 'ConnectionError'
 
     def set_container(self, container):
+        """Set the container object"""
         self.container = container
 
     def get_container(self):
+        """Fetch the container object"""
         return self.container
 
 
@@ -377,8 +393,8 @@ def start_docker_containers(docker_containers, network_name='demo'):
     # Test if Docker Service is available
     try:
         d_client.version()
-    except Exception:
-        raise EnvironmentError("Docker Service not found")
+    except Exception as e:
+        _raise(EnvironmentError("Docker Service not found"), e)
 
     for target in docker_containers:
         assert isinstance(target, DockerContainer)
@@ -436,11 +452,26 @@ def start_docker_containers(docker_containers, network_name='demo'):
         ))
 
 
+def strip_snapshot(java_version):
+    """
+    Strips the -SNAPSHOT suffix from a version string
+
+    Args:
+        java_version (str): the version string
+    """
+    assert isinstance(java_version, six.string_types)
+    return re.sub("-SNAPSHOT", "", java_version)
+
+
 def check_version(base, comparator=None, service='nifi'):
     """
     Compares version 'a' against either version 'b', or the version of the
     currently connected service instance.
 
+    Since NiFi is java, it may return a version with -SNAPSHOT as part of it.
+    As such, that will be stripped from either the comparator version or
+    the version returned from NiFi
+
     Args:
         base (str): The base version for the comparison test
         comparator (optional[str]): The version to compare against
@@ -457,12 +488,11 @@ def check_version(base, comparator=None, service='nifi'):
     ver_a = version.parse(base)
     if comparator:
         # if b is set, we compare the passed versions
+        comparator = strip_snapshot(comparator)
         ver_b = version.parse(comparator)
     else:
         # if b not set, we compare a against the connected nifi instance
-        ver_b = version.parse(
-            nipyapi.system.get_nifi_version_info().ni_fi_version
-        )
+        ver_b = version.parse(strip_snapshot(nipyapi.system.get_nifi_version_info().ni_fi_version))
     if ver_b > ver_a:
         return -1
     if ver_b < ver_a:
@@ -470,6 +500,26 @@ def check_version(base, comparator=None, service='nifi'):
     return 0
 
 
+def enforce_min_ver(min_version, bool_response=False):
+    """
+    Raises an error if target NiFi environment is not minimum version
+    Args:
+        min_version (str): Version to check against
+        bool_response (bool): If True, will return True instead of
+         raising error
+
+    Returns:
+        (bool) or (NotImplementedError)
+    """
+    if check_version(min_version) == 1:
+        if not bool_response:
+            raise NotImplementedError(
+                "This function is not available "
+                "before NiFi version " + str(min_version))
+        return True
+    return False
+
+
 def infer_object_label_from_class(obj):
     """
     Returns the expected STRING label for an object class required by certain
@@ -532,4 +582,20 @@ def rest_exceptions():
         yield
     except (nipyapi.nifi.rest.ApiException,
             nipyapi.registry.rest.ApiException) as e:
-        raise ValueError(e.body)
+        _raise(ValueError(e.body), e)
+
+
+def exception_handler(status_code=None, response=None):
+    """Simple Function wrapper to handle HTTP Status Exceptions"""
+    def func_wrapper(f):
+        @wraps(f)
+        def wrapper(*args, **kwargs):
+            try:
+                return f(*args, **kwargs)
+            except (nipyapi.nifi.rest.ApiException,
+                    nipyapi.registry.rest.ApiException) as e:
+                if status_code is not None and e.status == int(status_code):
+                    return response
+                _raise(ValueError(e.body), e)
+        return wrapper
+    return func_wrapper
